﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Input;
using Infragistics.Documents;
using Microsoft.Win32;
using SyntaxEditorDemo.SL.Core;
using System.IO;

namespace SyntaxEditorDemo
{
    public class SyntaxEditorViewModel : INotifyPropertyChanged
    {
        FileInfo _fileInfo;

        private TextDocument _document;
        public TextDocument Document
        {
            get { return _document; }
            set
            {
                _document = value;
                NotifyPropertyChanged("Document");
            }
        }

        private bool _isDirty = false;
        public bool IsDirty
        {
            get { return _isDirty; }
            set
            {
                _isDirty = value;
                NotifyPropertyChanged("IsDirty");
            }
        }

        private bool _isNew = false;
        public bool IsNew
        {
            get { return _isNew; }
            set
            {
                _isNew = value;
                NotifyPropertyChanged("IsNew");
            }
        }

        public ICommand OpenCommand { get; set; }
        public ICommand SaveCommand { get; set; }

        public SyntaxEditorViewModel()
        {
            CreateNewDocument();

            OpenCommand = new RelayCommand(OpenDocument);
            SaveCommand = new RelayCommand(SaveDocument, CanSaveDocument);
        }

        void Document_TextChanged(object sender, TextChangedEventArgs e)
        {
            IsDirty = true;
        }

        private void CreateNewDocument()
        {
            if (Document != null)
                Document.TextChanged -= Document_TextChanged;

            Document = new TextDocument();
            Document.Language = new CSharpLanguage();
            Document.TextChanged += Document_TextChanged;
            IsNew = true;
        }

        private void OpenDocument()
        {
            // Show a file open dialog with special handling for WPF and Silverlight 
            OpenFileDialog dialog = new OpenFileDialog();

            dialog.FilterIndex = 1;
            dialog.Filter = "C# Files |*.cs";

            bool? open = dialog.ShowDialog();
            if (open.HasValue && open.Value == true)
            {
                // Load the file into the TextDocument.
                try
                {
                    _fileInfo = new FileInfo(dialog.FileName);

                    using (var stream = _fileInfo.OpenRead())
                    {
                        Document.Load(stream);
                    }                    

                    IsNew = false;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(string.Format("Error '{0}' while loading file.", "Error Loading File", ex.Message), "Error Loading File", MessageBoxButton.OK);
                    return;
                }
            }
        }

        private void SaveDocument()
        {
            if (IsNew)
                SaveNewDocument();
            else
                SaveCurrentDocument(_fileInfo.OpenWrite());

            IsNew = false;
            IsDirty = false;
        }

        private bool CanSaveDocument()
        {
            return IsNew || IsDirty;
        }

        private void SaveNewDocument()
        {
            SaveFileDialog saveDialog = new SaveFileDialog();
            saveDialog.Filter = "C# Files |*.cs";

            bool? result = saveDialog.ShowDialog();
            if (result.Value)
            {
                SaveCurrentDocument(saveDialog.OpenFile());
                _fileInfo = new FileInfo(saveDialog.FileName);
            }
        }

        private void SaveCurrentDocument(Stream fileStream)
        {
            try
            {
                using (StreamWriter streamWriter = new StreamWriter(fileStream))
                {
                    streamWriter.Write(Document.CurrentSnapshot.GetText());
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(string.Format("Error '{0}' while saving file", "Error Saving File", ex.Message), "Error Saving File", MessageBoxButton.OK);
            }
        }

        #region INotifyPropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;
        protected void NotifyPropertyChanged(string propertyName)
        {
            var handler = PropertyChanged;
            if (handler != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion //INotifyPropertyChanged
    }
}
